package cerebral.colors;

import java.awt.BasicStroke;
import java.awt.Color;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.animate.ColorAnimator;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ColorLib;
import prefuse.util.StrokeLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cerebral.Cerebral;
import cerebral.layout.LabelLayout;
import cytoscape.visual.LineType;

public class ColorManager {
    // red on white
    public final static int Highlight1 = ColorLib.hsb(0, 0.9f, 0.8f);

    private final static int Highlight1Inverse = ColorLib.hsb(0, 0.0f, 1.0f);

    private final static int Highlight4 = ColorLib.hsb(0.08f, 0.9f, 1.0f);

    // white on green
    public final static int Highlight2 = ColorLib.hsb(0.3f, 0.9f, 0.8f);

    private final static int Highlight2Inverse = ColorLib.hsb(0, 0.0f, 1.0f);

   
    private final static float[] hsbVals = new float[3];

    public LabelColorAction labelColorAction = new LabelColorAction(
            Cerebral.LABELS);

    public LabelBGColorAction labelBGColorAction = new LabelBGColorAction(
            Cerebral.LABELS);

    public MyStrokeAction edgeStrokeAction = new MyStrokeAction(
            Cerebral.EDGES);

    public boolean pathColoring = false;

    private Visualization vis;

    private ActionList otherColorRules;

    private ActionList cytoscapeFill;

    public ColorManager() {

    }

    public void createColorRules(Visualization vis, LabelLayout labelLayout,
            Table edges) {

        FillNodeAction fillNodeAction = new FillNodeAction(Cerebral.NODES);
        BorderAction borderAction = new BorderAction(Cerebral.NODES);

        // create an action list containing all color assignments
        cytoscapeFill = new ActionList();
        cytoscapeFill.add(fillNodeAction);
        cytoscapeFill.add(borderAction);
        

        otherColorRules = new ActionList();
        otherColorRules.add(edgeStrokeAction);
        otherColorRules.add(new EdgeFadeColorAction(Cerebral.EDGES, edges));
        otherColorRules.add(labelLayout);
//        otherColorRules.add(new RepaintAction());

        ActionList color = new ActionList();
        color.add(cytoscapeFill);
        color.add(otherColorRules);
       
        ActionList animate = new ActionList(200);
        
        animate.add(new ColorAnimator(Cerebral.NODES, VisualItem.FILLCOLOR));
        animate.add(new RepaintAction());
        
 
        vis.putAction("color", color);
        vis.putAction("animate", animate);
        this.vis = vis;
    }

    
    public class FillNodeAction extends ColorAction {
        private FillNodeAction(String group) {
            super(group, VisualItem.FILLCOLOR);
        }

        public int getColor(VisualItem item) {
            NodeItem node = (NodeItem) item;
            TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
            boolean fullColor = false;
            if (selectedItems.containsTuple(node)) {
                fullColor = true;
            }
            if (node.isHover() || node.getBoolean(VisualItem.HIGHLIGHT)
                    || node.isInGroup(Cerebral.SINGLE_HIGHLIGHT)
                    || node.isInGroup(Cerebral.NEIGHB_SELECT))
                fullColor = true;

            if (!fullColor) {
                return ColorLib.rgba(0,0,0,0);
            } else {

                Color cytoColor = (Color) node.get(Cerebral.FILL_COLOR);
                if (cytoColor == null) {
                    return 0;
                }
                return ColorLib.color(cytoColor);
            }

        }

    }

    public class BorderAction extends ColorAction {
        private BorderAction(String group) {
            super(group, VisualItem.STROKECOLOR);
        }

        public int getColor(VisualItem item) {
            NodeItem node = (NodeItem) item;
            TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
            boolean fullColor = false;
            if (selectedItems.containsTuple(node)) {
                fullColor = true;
            }
            if (node.isHover() || node.getBoolean(VisualItem.HIGHLIGHT)
                    || node.isInGroup(Cerebral.SINGLE_HIGHLIGHT)
                    || node.isInGroup(Cerebral.NEIGHB_SELECT))
                fullColor = true;

            if (!fullColor) {
                return ColorLib.rgba(0,0,0,45);
            }

            if (node.getBoolean(VisualItem.HOVER)) {
                return Highlight1;
            }
            if (node.isInGroup(Cerebral.SINGLE_HIGHLIGHT)) {
                return Highlight2;
            }
            if (node.isInGroup(Cerebral.NEIGHB_SELECT)) {
                return Highlight4;
            }

            Color c = (Color) item.get(Cerebral.BORDER_COLOR);
            if (c == null) {
                return ColorLib.gray(60);
            }
            return ColorLib.color(c);
        }

    }

    public class LabelColorAction extends ColorAction {
        private LabelColorAction(String group) {
            super(group, VisualItem.TEXTCOLOR);
        }

        public int getColor(VisualItem item) {
            DecoratorItem di = (DecoratorItem) item;
            NodeItem node = (NodeItem) di.getDecoratedItem();

            if (node.getBoolean(VisualItem.HOVER)) {
                return Highlight1Inverse;
            }
            if (node.isInGroup(Cerebral.SINGLE_HIGHLIGHT)) {
                return Highlight2Inverse;
            }
            if (node.isInGroup(Cerebral.NEIGHB_SELECT)) {
                return Highlight1Inverse;
            }
            Color c = (Color) node.get(Cerebral.TEXT_COLOR);
            if (c == null) {
                return ColorLib.gray(60);
            }
            return ColorLib.color(c);

        }
    }

    public static int changeSaturationTo(Color c, float saturation) {
        if (c.getRed() == c.getGreen() && c.getGreen() == c.getBlue()) {
            // a grey scale. Change brightness instead of saturation
            return ColorLib.gray(255 - (int) (saturation * 255));
        } else {
            Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsbVals);
            return ColorLib.hsb(hsbVals[0], saturation, hsbVals[2]);
        }
    }

    public static int increasedContrast(int color) {
        return increasedContrast(color, 0.2f);
    }
    
    public static int increasedContrast (int color, float depth){
        Color c = ColorLib.getColor(color);
        Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsbVals);
        float deeperSaturation = hsbVals[1] * (1.0f + depth);
        if (deeperSaturation > 1.0f) {
            deeperSaturation = 1.0f;
        }
        float lessBright = hsbVals[2] * (1.0f - depth);
        if (lessBright < 0.0f) {
            lessBright = 0.0f;
        }
        return ColorLib.hsb(hsbVals[0], deeperSaturation, lessBright);  
    }

    public class LabelBGColorAction extends ColorAction {
        private LabelBGColorAction(String group) {
            super(group, VisualItem.FILLCOLOR);
        }

        public int getColor(VisualItem item) {
            DecoratorItem di = (DecoratorItem) item;
            NodeItem node = (NodeItem) di.getDecoratedItem();

            if (node.getBoolean(VisualItem.HOVER)) {
                return Highlight1;
            }
            if (node.isInGroup(Cerebral.SINGLE_HIGHLIGHT)) {
                return Highlight2;
            }
            if (node.isInGroup(Cerebral.NEIGHB_SELECT))
                return Highlight4;


            Color bgColor;

            if (vis != null) {
                Display d = vis.getDisplay(0);
                bgColor = d.getBackground();
            } else
                bgColor = ColorLib.getGrayscale(0);

            return ColorLib.rgba(bgColor.getRed(), bgColor.getGreen(), bgColor
                    .getBlue(), 200);

        }
    }

    public class MyStrokeAction extends StrokeAction {

        MyStrokeAction(String group) {
            super(group);
        }

        public BasicStroke getStroke(VisualItem item) {

            TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
            boolean fullColor = false;
            if (selectedItems.containsTuple(item)) {
                fullColor = true;
            }
            float width = 1.0f;

            LineType lt = (LineType) item.get(Cerebral.LINE_TYPE);
            BasicStroke basicStroke;
            if (lt != null) {
                basicStroke = (BasicStroke) lt.getStroke();
            } else {
                basicStroke = StrokeLib.getStroke(width, BasicStroke.CAP_ROUND,
                        BasicStroke.JOIN_ROUND);
            }
            width = basicStroke.getLineWidth();
            if (!fullColor) {
                return StrokeLib.getStroke((float) (basicStroke.getLineWidth()), BasicStroke.CAP_ROUND,
                        BasicStroke.JOIN_ROUND, basicStroke.getMiterLimit(),
                        basicStroke.getDashArray(), basicStroke.getDashPhase());
            } else {

                if (item.isInGroup(Cerebral.SINGLE_HIGHLIGHT)) {
                    return StrokeLib.getStroke((float) (1.8 *basicStroke.getLineWidth()), BasicStroke.CAP_ROUND,
                        BasicStroke.JOIN_ROUND, basicStroke.getMiterLimit(),
                        basicStroke.getDashArray(), basicStroke.getDashPhase());
                } else if (item.getBoolean(VisualItem.HOVER)) {
                    return StrokeLib.getStroke((float) (1.5 *basicStroke.getLineWidth()), BasicStroke.CAP_ROUND,
                            BasicStroke.JOIN_ROUND, basicStroke.getMiterLimit(),
                            basicStroke.getDashArray(), basicStroke.getDashPhase());
                } else if (item.getBoolean(VisualItem.HIGHLIGHT)) {
                    return StrokeLib.getStroke((float) (1.5 *basicStroke.getLineWidth()), BasicStroke.CAP_ROUND,
                        BasicStroke.JOIN_ROUND, basicStroke.getMiterLimit(),
                        basicStroke.getDashArray(), basicStroke.getDashPhase());
                } else if (item.isInGroup(Cerebral.NEIGHB_SELECT)) {
                    return StrokeLib.getStroke((float) (1.5 *basicStroke.getLineWidth()), BasicStroke.CAP_ROUND,
                        BasicStroke.JOIN_ROUND, basicStroke.getMiterLimit(),
                        basicStroke.getDashArray(), basicStroke.getDashPhase());
                } else {
                    return StrokeLib.getStroke((float) (basicStroke.getLineWidth()), BasicStroke.CAP_ROUND,
                            BasicStroke.JOIN_ROUND, basicStroke.getMiterLimit(),
                            basicStroke.getDashArray(), basicStroke.getDashPhase());
                }
            }

        }
    }

    public class EdgeFadeColorAction extends ColorAction {

        private Table edges;

        public EdgeFadeColorAction(String group, Table edges) {
            super(group, VisualItem.STROKECOLOR);
            this.edges = edges;
        }

        public int getColor(VisualItem item) {
            EdgeItem edge = (EdgeItem) item;
            TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
            boolean fullColor = false;
            if (selectedItems.containsTuple(item)) {
                fullColor = true;
            } else if (item.isHighlighted() || item.isHover()
                    || item.isInGroup(Cerebral.NEIGHB_SELECT)) {
                fullColor = true;
            }
            if (!fullColor) {
                return increasedContrast(ColorLib.color(m_vis.getDisplay(0)
                        .getBackground()));
            } else {
                if (item.getBoolean(VisualItem.HOVER))
                    return Highlight1;
                if (item.isInGroup(Cerebral.SINGLE_HIGHLIGHT))
                    return Highlight2;
             
                if (item.getBoolean(VisualItem.HIGHLIGHT))
                    return Highlight1;
                if (item.isInGroup(Cerebral.NEIGHB_SELECT))
                    return Highlight1;

                ColumnMetadata edgeLengthMetadata = edges
                        .getMetadata(Cerebral.EDGE_LENGTH);
                int maxRow = edgeLengthMetadata.getMaximumRow();
                float maxLength = edges.getInt(maxRow, Cerebral.EDGE_LENGTH);

                Color c = (Color) edge.get(Cerebral.FILL_COLOR);
                if (c == null) {
                    c = ColorLib.getColor(255, 255, 255);
                }

                int edgeLength = edge.getInt(Cerebral.EDGE_LENGTH);
                double normalizedLength = 1 - (edgeLength / maxLength);
                double alphaFactor = normalizedLength * normalizedLength
                        * normalizedLength * normalizedLength;

                return changeSaturationTo(c,
                        (float) ((100 + 155 * alphaFactor) / 255.0f));
            }
        }
    }

    public boolean isSpecialColoring(EdgeItem item) {
        if (item.isInGroup(Cerebral.SINGLE_HIGHLIGHT))
            return true;
        if (item.getBoolean(VisualItem.HOVER))
            return true;
        if (item.isInGroup(Cerebral.NEIGHB_SELECT))
            return true;
        if (item.isInGroup(Cerebral.SELECTED))
            return true;

        return false;

    }

    public void setCytoNodeSelected(Color nodeSelectionColor) {

    }

    public void setCytoEdgeSelected(Color edgeSelectionColor) {

    }

    public void setCytoscapeFill(ActionList fillColor) {
        
        ActionList color = new ActionList();
        if (fillColor != null) {
            color.add(fillColor);
        } else {
            color.add(cytoscapeFill);
        }
        color.add(otherColorRules);
        vis.putAction("color", color);
        
    }

}
